/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade;

import java.util.Collection;
import java.util.Map;
import java.util.Set;

import org.ow2.bonita.facade.exception.ActivityNotFoundException;
import org.ow2.bonita.facade.exception.InstanceNotFoundException;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.facade.exception.TaskNotFoundException;
import org.ow2.bonita.facade.exception.VariableNotFoundException;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.ProcessInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;

/**
 * Getters on workflow recorded data.<br>
 * Operations in this API applies to main entities managed by Nova Bonita such as:<br>
 * packages, processes, process instances, activities, tasks.<br>
 * Returned records are issued from recorded runtime informations (both runtime and archived workflow instances).
 * @see org.ow2.bonita.definition.RoleMapper
 * @see org.ow2.bonita.definition.PerformerAssign
 * @see org.ow2.bonita.definition.Hook
 * @author Pierre Vigneras, Charles Souillard
 */
public interface QueryRuntimeAPI {

  /**
   * Returns all records of instance.<br>
   * An empty set is returned if no instance is found.
   * @return a set containing all instance records.
   * @throws BonitaInternalException if an exception occurs.
   */
  Set<ProcessInstance> getProcessInstances();

  /**
   * Returns the record of the instance with the given UUID.
   * @param instanceUUID the instance UUID.
   * @return the record of the instance with the given UUID.
   * @throws InstanceNotFoundException if no instance has been found with the given instance UUID.
   * @throws BonitaInternalException if an exception occurs.
   */
  ProcessInstance getProcessInstance(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException;

  /**
   * Returns all records of instance for the given process processDefinitionUUID.<br>
   * An empty set is returned if no instance is found.
   * @param processUUID the process definition UUID.
   * @return a set containing all instance records.
   * @throws ProcessNotFoundException if no process has been found with the given process UUID.
   * @throws BonitaInternalException if an exception occurs.
   */
  Set<ProcessInstance> getProcessInstances(ProcessDefinitionUUID processUUID) throws ProcessNotFoundException;

  /**
   * Returns the record of the activity with the given activity UUID.
   * @param activityUUID the activity UUID.
   * @return the activity record with the given instance UUID and activity id.
   * @throws InstanceNotFoundException if no instance has been found with the given instance UUID.
   * @throws ActivityNotFoundException if no activity has been found with the given activity id.
   * @throws BonitaInternalException if an exception occurs.
   */
  ActivityInstance<ActivityBody> getActivityInstance(ActivityInstanceUUID activityUUID) throws ActivityNotFoundException;

  /**
   * Returns all records of activity for the given process instance UUID.
   * @param instanceUUID the instance UUID.
   * @return the unordered set containing activity records.
   * @throws InstanceNotFoundException if no instance has been found with the given instance UUID.
   * @throws BonitaInternalException if an exception occurs.
   */
  Set<ActivityInstance<ActivityBody>> getActivityInstances(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException;

  //all iterations
  /**
   * Returns records for all iterations and multi-instantiations that should happend for the given
   * process instance UUID and activity Id.<br>
   * An empty set is returned if no instance is found.
   * @param instanceUUID the instance UUID.
   * @param activityId the activity Id (as specified into the XPDL file).
   * @return the unordered set containing activity records.
   * @throws InstanceNotFoundException if no instance has been found with the given instance UUID.
   * @throws BonitaInternalException if an exception occurs.
   */
  Set<ActivityInstance<ActivityBody>> getActivityInstances(ProcessInstanceUUID instanceUUID,
      String activityId) throws InstanceNotFoundException, ActivityNotFoundException;

  /**
   * Returns the record of the task with the given task UUID.
   * @param taskUUID the task UUID.
   * @return the record of the task with the given task UUID.
   * @throws TaskNotFoundException if no task has been found with the given task UUID.
   * @throws BonitaInternalException if an exception occurs.
   */
  ActivityInstance<TaskInstance> getTask(TaskUUID taskUUID) throws TaskNotFoundException;

  /**
   * Returns all records of task for the given process instance UUID.
   * @param instanceUUID the instance UUID.
   * @return a set containing all task records for the instance.
   * @throws InstanceNotFoundException if no instance has been found with the given instance UUID.
   * @throws BonitaInternalException if an exception occurs.
   */
  Set<ActivityInstance<TaskInstance>> getTasks(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException;

  /**
   * Obtains the user tasks with state either READY or EXECUTING or SUSPENDED or FINISHED
   * for the given instance and the authenticated user.<br>
   * @param instanceUUID the instance UUID.
   * @param taskState the {@link org.ow2.bonita.facade.runtime.TaskState state} of the task.
   * @return a collection of task records. If no tasks are found, an empty collection is returned.
   * @throws InstanceNotFoundException if no instance record is found with the given instance UUID.
   * @throws BonitaInternalException if an other exception occurs.
   */
  Collection<ActivityInstance<TaskInstance>> getTaskList(ProcessInstanceUUID instanceUUID, TaskState taskState)
    throws InstanceNotFoundException;

  /**
   * Obtains the tasks with state either READY or EXECUTING or SUSPENDED or FINISHED for the given instance
   * and for the given user.<br>
   * If the task has been assigned to a user, only this user can get the task into the returned list.
   * Otherwise all the users that believe to the candidate list can get the task.
   * @param instanceUUID the instance UUID.
   * @param userId the userId for which the tasks are searched.
   * @param taskState the {@link org.ow2.bonita.facade.runtime.TaskState state} of the task.
   * @return a collection of task records. If no tasks are found, an empty collection is returned.
   * @throws InstanceNotFoundException if no instance record is found with the given instance UUID.
   * @throws BonitaInternalException if an other exception occurs.
   */
  Collection<ActivityInstance<TaskInstance>> getTaskList(ProcessInstanceUUID instanceUUID, String userId, TaskState taskState)
    throws InstanceNotFoundException;

  /**
   * Obtains the user tasks with state either READY or EXECUTING or SUSPENDED or FINISHED
   * for the authenticated user.<br>
   * If the task has been assigned to a user, only this user can get the task into the returned list.<br>
   * Otherwise all the users that believe to the candidate list can get the task.
   * @param taskState the {@link org.ow2.bonita.facade.runtime.TaskState state} of the task.
   * @return a collection of task records. If no tasks are found, an empty collection is returned.
   * @throws BonitaInternalException if an other exception occurs.
   */
  Collection<ActivityInstance<TaskInstance>> getTaskList(TaskState taskState);

  /**
   * <p>
   * Obtains the user tasks with state either READY or EXECUTING or SUSPENDED or FINISHED
   * for the given user.<br>
   * If the task has been assigned to a user, only this user can get the task into the returned list.<br>
   * Otherwise all the users that believe to the candidate list can get the task.
   * @param userId the userId for which the tasks are searched.
   * @param taskState the {@link org.ow2.bonita.facade.runtime.TaskState state} of the task.
   * @return a collection of task records. If no tasks are found, an empty collection is returned.
   * @throws BonitaInternalException if an other exception occurs.
   */
  Collection<ActivityInstance<TaskInstance>> getTaskList(String userId, TaskState taskState);

  /**
   * Obtains a variable defined as local to the activity for the given activity UUID and variable Id.
   * The activity should either be executed or currently pointed by the a process execution.<br>
   * @param activityUUID the activity UUID.
   * @param variableId the value of the Id attribute of the DataField element defined into the XPDL file.
   * @return the variable object (can be: an {@link org.ow2.bonita.facade.runtime.var.Enumeration Enumeration}
   * or a plain {@link String}).
   * @throws ActivityNotFoundException if this activity is not pointed by a process execution
   * and the execution informations for this activity has not been recorded.
   * @throws VariableNotFoundException if no variable is found with the given id.
   * @throws BonitaInternalException if an other exception occurs.
   */
  Object getActivityInstanceVariable(ActivityInstanceUUID activityUUID,
      String variableId) throws ActivityNotFoundException, VariableNotFoundException;

  /**
   * Obtains the variables defined as local to the activity for the given activity UUID.<br>
   * An empty map is returned if no variable is found.
   * @param activityUUID the activity UUID.
   * @return the map of activity variables where key is the variable id and value is the variable object.
   * @throws ActivityNotFoundException if this activity is not pointed by a process execution
   * and the execution informations for this activity has not been recorded.
   * @throws BonitaInternalException if an other exception occurs.
   */
  Map<String, Object> getActivityInstanceVariables(ActivityInstanceUUID activityUUID)throws ActivityNotFoundException;

  /**
   * Obtains a variable for the given activity and variable Id.
   * This variable could be local to the activity or global to the process.
   * The activity should either be executed or currently pointed by the a process execution.
   * @param activityUUID the activity UUID.
   * @param variableId the value of the Id attribute of the DataField element defined into the XPDL file.
   * @return the variable object (can be: an {@link org.ow2.bonita.facade.runtime.var.Enumeration Enumeration}
   * or a plain String})
   * @throws ActivityNotFoundException if this activity is not pointed by a process execution
   * and the execution informations for this activity has not been recorded.
   * @throws VariableNotFoundException if no variable is found with the given id.
   * @throws BonitaInternalException if an other exception occurs.
   */
  Object getVariable(ActivityInstanceUUID activityUUID, String variableId)
    throws ActivityNotFoundException, VariableNotFoundException;

  /**
   * Obtains the activity variables (including global process and local activity variables)
   * for the given activity UUID.<br>
   * An empty map is returned if no variable is found.
   * @param activityUUID the activity UUID.
   * @return the map of activity variables where key is the variable id and value is the variable object.
   * @throws ActivityNotFoundException if this activity is not pointed by a process execution
   * and the execution informations for this activity has not been recorded.
   * @throws InstanceNotFoundException if no recorded informations is found with the given instance UUID.
   * @throws BonitaInternalException if an other exception occurs.
   */
  Map<String, Object> getVariables(ActivityInstanceUUID activityUUID)
    throws InstanceNotFoundException, ActivityNotFoundException;

  /**
   * Obtains a process variable for the given process instance UUID and variable Id.
   * @param instanceUUID the instance UUID.
   * @param variableId the variable id.
   * @return the variable object
   * (can be: {@link org.ow2.bonita.facade.runtime.var.Enumeration} if the XPDL type is Enumeration<br>
   * or standard classes from the JDK for supported basic XPDL
   * {@link org.ow2.bonita.facade.def.dataType.BasicTypeDefinition.Type types}).
   * @throws InstanceNotFoundException if no instance has found with the given instance UUID.
   * @throws VariableNotFoundException if no variable is found with the given id.
   * @throws BonitaInternalException if an other exception occurs.
   */
  Object getProcessInstanceVariable(ProcessInstanceUUID instanceUUID,
      String variableId) throws InstanceNotFoundException, VariableNotFoundException;

  /**
   * Obtains the process variables for the given process instance UUID.
   * An empty map is returned if no process variable is found.
   * @param instanceUUID the instance UUID.
   * @return the map of process variables where key is the variable id and value is the variable object.
   * @throws InstanceNotFoundException if no instance has been found with the given instance UUID.
   * @throws BonitaInternalException if an other exception occurs.
   */
  Map<String, Object> getProcessInstanceVariables(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException;
}
